#!/usr/bin/env python
# -*- coding: utf-8 -*-
# encoding: utf-8

from django.utils.translation import ugettext_lazy as _
from django.db.models import Max, Min, Count, F, Q
from django.contrib.contenttypes.models import ContentType
import unicodedata


def getFieldVerboseName(qs, field):
    try:
        # Find qs non related model fields
        if '__' not in field:
            return _(qs.model._meta.get_field(field).verbose_name)

        # Find the related model field
        else:
            if '__id' in field:
                field = field.replace('__id', '')
            else:
                related_model, field = field.split('__')[-2], field.split('__')[-1]
        try:
            return _(qs.model._meta.get_field(related_model).rel.to._meta.get_field(field).verbose_name)
        except:
            ct = ContentType.objects.get(model=related_model)
            return get_model(ct.app_label, ct.model)._meta.get_field(field).verbose_name
    except:
        return field.replace('_', ' ').capitalize()


# Gets the translation of a field or related field name
def getFieldType(qs, field):
    try:
        # Find qs non related model fields
        if '__' not in field:
            return _(qs.model._meta.get_field(field).get_internal_type())
        # Find the related model field
        else:
            if '__id' in field:
                field = field.replace('__id', '')
            else:
                related_model, field = field.split('__')[-2], field.split('__')[-1]
        try:
            return _(qs.model._meta.get_field(related_model).rel.to._meta.get_field(field).get_internal_type())
        except:
            ct = ContentType.objects.get(model=related_model)
            return get_model(ct.app_label, ct.model)._meta.get_field(field).get_internal_type()
    except:
        return field.replace('_', ' ').capitalize()


def mergePerson(queryset, request_data, config):
    if config == 'Client':
        type_id = 'client_type_id'
        type_id_in = 'client_type_id__in'
        person = 'client'
        person_group = 'clientgroup'
        pg = 'cg_'
        p = 'c_'
    elif config == 'ClientEntry':
        type_id = 'cliententry_type_id'
        type_id_in = 'cliententry_type_id__in'
        person = 'client'
        person_group = 'clientgroup'
        pg = 'cg_'
        p = 'c_'
    elif config == 'ClientEntry':
        type_id = 'cliententry_type_id'
        type_id_in = 'cliententry_type_id__in'
        person = 'client'
        person_group = 'clientgroup'
        pg = 'cg_'
        p = 'c_'
    elif config == 'Provider':
        type_id = 'provider_type_id'
        type_id_in = 'provider_type_id__in'
        person = 'provider'
        person_group = 'providergroup'
        pg = 'pg_'
        p = 'p_'
    elif config == 'ProviderPayout':
        type_id = 'providerpayout_type_id'
        type_id_in = 'providerpayout_type_id__in'
        person = 'provider'
        person_group = 'providergroup'
        pg = 'pg_'
        p = 'p_'
    else:
        return queryset, request_data
    if type_id in request_data:
        if request_data[type_id][1] != 'NULL':
            list_person_invoice = []
            list_persongroup_invoice = []
            if request_data[type_id][0] == type_id:
                request_data[type_id] = (unicode(type_id_in), [request_data[type_id][1]])
            if request_data[type_id][0] == type_id_in:
                for i in range(0, len(request_data[type_id][1])):
                    if type(request_data[type_id][1][i]) == unicode:
                        item_client = unicodedata.normalize('NFKD', request_data[type_id][1][i]).encode('ascii', 'ignore')
                    else:
                        item_client = request_data[type_id][1][i]

                    if pg in item_client:
                        request_data[type_id][1][i] = request_data[type_id][1][i].replace(pg, '')
                        list_persongroup_invoice.append(request_data[type_id][1][i])
                    if p in item_client:
                        request_data[type_id][1][i] = request_data[type_id][1][i].replace(p, '')
                        list_person_invoice.append(request_data[type_id][1][i])
                # del request_data[type_id]
                if config == 'Client':
                    queryset = queryset.filter(Q(client_type_id__in=list_person_invoice, client_contenttype__model=person) | Q(client_type_id__in=list_persongroup_invoice, client_contenttype__model=person_group))
                elif config == 'ClientEntry':
                    queryset = queryset.filter(Q(cliententry_type_id__in=list_person_invoice, cliententry_contenttype__model=person) | Q(cliententry_type_id__in=list_persongroup_invoice, cliententry_contenttype__model=person_group))
                elif config == 'Provider':
                    queryset = queryset.filter(Q(provider_type_id__in=list_person_invoice, provider_contenttype__model=person) | Q(provider_type_id__in=list_persongroup_invoice, provider_contenttype__model=person_group))
                elif config == 'ProviderPayout':
                    queryset = queryset.filter(Q(providerpayout_type_id__in=list_person_invoice, providerpayout_contenttype__model=person) | Q(providerpayout_type_id__in=list_persongroup_invoice, providerpayout_contenttype__model=person_group))
                else:
                    pass

    return queryset, request_data



#!/usr/bin/env python
# encoding: utf-8

import csv
from django.http import HttpResponse, StreamingHttpResponse
from decimal import Decimal
from datetime import date, datetime
from sws_tags.sws_utils.common_utils import *
from sws_tags.sws_utils.messages import *
from sws_tags.sws_utils.cube import *
from sws_tags.sws_utils.pdf_utils import *
import os.path
import traceback

# FOR EXCEL
try:
    import cStringIO as StringIO
except ImportError:
    import StringIO

from xlsxwriter.workbook import Workbook

# FOR EXCEL GENERATING
import xlwt
ezxf = xlwt.easyxf

from sws_tags.sws_utils.common_utils import *
from sws_tags.sws_utils.messages import *
from sws_tags.sws_utils.cube import *


def getRedirectURL(param_export_file, request, dst_directory='/media'):
    ''' Create a redirect URL depending on the request and wether it is secure or not'''
    if request is not None:
        redirect_url = request.get_host() + '/' + dst_directory + param_export_file['filename']
        redirect_url = redirect_url.replace('//', '/')
        if request.is_secure():
            redirect_url = 'http://' + redirect_url
        else:
            redirect_url = 'https://' + redirect_url
    else:
        redirect_url = dst_directory + param_export_file['filename']

    return redirect_url


class Echo(object):
    """An object that implements just the write method of the file-like
    interface.
    """
    def write(self, value):
        """Write the value by returning it, instead of storing in a buffer."""
        return value


def rounding(cifra, digitos=0, format='Round_up'):
    """Rutina par rounding de cifras decimales como para uso en contabilidad"""
    # Symmetric Arithmetic Rounding for decimal numbers
    if type(cifra) != decimal.Decimal:
        cifra = decimal.Decimal(str(cifra))

    if format == 'Default':
        result = cifra.quantize(decimal.Decimal("1") / (decimal.Decimal('10') ** digitos), decimal.ROUND_HALF_UP)
    elif format == 'Round_up':
        result = cifra.quantize(decimal.Decimal("1") / (decimal.Decimal('10') ** digitos), decimal.ROUND_UP)
    elif format == 'Round_down':
        result = cifra.quantize(decimal.Decimal("1") / (decimal.Decimal('10') ** digitos), decimal.ROUND_DOWN)

    return result


def roundingBill(data, conf):
    new_data = []
    new_total = 0
    new_sum_minutes = 0

    for r in data:
        new_r = {}
        for i in r:
            if i not in conf:
                new_r[i] = r[i]
            elif i == 'sum_total_minutes':
                new_r[i] = rounding(r[i], conf[i][0], conf[i][1])

        for c in conf:
            if c == 'sum_total_income':
                new_sum_total_income = new_r['income'] * new_r['sum_total_minutes']
                new_r[c] = rounding(new_sum_total_income, conf[c][0], conf[c][1])

        new_data.append(new_r)
        new_total = new_total + new_r['sum_total_income']
        new_sum_minutes = new_sum_minutes + new_r['sum_total_minutes']

    sum_data = {}
    sum_data['sum_total_income'] = new_total
    sum_data['sum_total_minutes'] = new_sum_minutes

    return new_data, sum_data


def roundingBillIncome(data, conf):
    new_data = []
    new_total = 0
    new_sum_minutes = 0

    for r in data:

        new_r = {}
        for i in r:
            if i not in conf:
                new_r[i] = r[i]
            elif i == 'minutes':
                new_r[i] = rounding(r[i], conf[i][0], conf[i][1])

        for c in conf:
            if c == 'total_euro':
                new_total_euro = new_r['rate_euro'] * new_r['minutes']
                new_r[c] = rounding(new_total_euro, conf[c][0], conf[c][1])

        new_data.append(new_r)
        new_total = new_total + new_r['total_euro']
        new_sum_minutes = new_sum_minutes + new_r['minutes']

    sum_data = {}
    sum_data['total_euro'] = new_total
    sum_data['minutes'] = new_sum_minutes
    return new_data, sum_data


def ExportFile(qs, param_export_file, request=None):
    def read_and_flush():
        output.seek(0)
        data = output.read()
        output.seek(0)
        output.truncate()
        return data

    def writeInStream():
            row = 0
            col = 0
            for h in headers:
                sheet.write(row, col, h, bold)
                col += 1
            row = 1
            col = 0
            for trr in queryset:
                for c in col_name:
                    if type(trr[c]) == datetime:
                        try:
                            trr[c] = param_export_file['tz'].normalize(trr[c]).strftime('%Y-%m-%d %H:%M:%S')
                        except:
                            trr[c] = str(trr[c])
                    sheet.write(row, col, trr[c])
                    col += 1
                row += 1
                col = 0
                data = read_and_flush()
                yield data

            book.close()
            data = read_and_flush()
            yield data

    def writeBillInStream():
        # //////////////////////////////////////////////   STYLE ///////////////////////////////////
        bold = book.add_format({'bold': 1})
        yellow_bg = book.add_format({'bg_color': '#FFFFFF'})
        grey_bg = book.add_format({'bg_color': '#F7F7F7'})

        grey_dark_bg_bold = book.add_format({'bg_color': '#D7D6D2', 'bold': 1})

        flesh_color_bg = book.add_format({'bg_color': '#FFE8E8'})
        flesh_color_bg_bold = book.add_format({'bg_color': '#FFE8E8', 'bold': 1})

        merge_format_border = book.add_format({
            'bold': 1,
            'align': 'center',
            'valign': 'vcenter',
            'fg_color': '#FFE8E8',
            })

        merge_format = book.add_format({
            'bold': 1,
            'align': 'center',
            'valign': 'vcenter',
            'fg_color': '#FFE8E8',
            })

        red_format = book.add_format({
            'bold': 1,
            'align': 'center',
            'valign': 'vcenter',
            'fg_color': '#FFB3B3',
            # 'border_color':'#FF3636',
            # 'border':6
            })

        # money = book.add_format({'num_format': '#.##'+param_export_file['data_bill']['type_money'], 'bold': 1})
        # money_grey_bg = book.add_format({'num_format': '#.##'+param_export_file['data_bill']['type_money'], 'bold': 1, 'bg_color': '#F7F7F7'})
        # money_grey_bg_0 = book.add_format({'num_format': '0.####'+param_export_file['data_bill']['type_money'], 'bold': 1, 'bg_color': '#F7F7F7'})
        # money_yellow_bg = book.add_format({'num_format': '#.####'+param_export_file['data_bill']['type_money'], 'bold': 1, 'bg_color': '#FBE067'})
        # money_yellow_bg_0 = book.add_format({'num_format': '0.####'+param_export_file['data_bill']['type_money'], 'bold': 1, 'bg_color': '#FBE067'})

        # minute_grey_bg = book.add_format({'num_format': '#.#', 'bg_color': '#F7F7F7'})
        # minute_grey_bg_0 = book.add_format({'num_format': '0.#', 'bg_color': '#F7F7F7'})
        # minute_yellow_bg = book.add_format({'num_format': '#.#', 'bg_color': '#FBE067'})
        # minute_yellow_bg_0 = book.add_format({'num_format': '0.#', 'bg_color': '#FBE067'})

        # date_format = book.add_format({'num_format': 'mmmm d yyyy'})
        # italic = book.add_format({'italic': True})

        # ///////////////////////////////////////////    DECLARATION /////////////////////////////////////////////
        pattern = xlwt.Pattern()  # Create the Pattern
        pattern.pattern = xlwt.Pattern.SOLID_PATTERN
        style = xlwt.XFStyle()  # Create the Pattern
        style.pattern = pattern

        queryset = param_export_file['data_bill']['raw_cdrts']
        total = param_export_file['data_bill']['cdrt_subtotal']

        col_name = ['destination', 'minutes', 'rate_euro', 'total_euro']
        headers = ['Destination', 'Total Minutes', 'EUR/min', 'Total']

        sheet.set_column('A:A', 5)
        sheet.set_column('B:B', 30)
        sheet.set_column('C:C', 15)
        sheet.set_column('D:D', 15)
        sheet.set_column('E:E', 15)

        sheet.set_column('G:G', 15)

        try:
            sheet.write(1, 1, param_export_file['data_bill']['issuing_company_name'], bold)
            sheet.write(2, 1, param_export_file['data_bill']['issuing_company_address'], bold)
            sheet.write(3, 1, param_export_file['data_bill']['issuing_company_CIF'], bold)
            sheet.insert_image('A1', param_export_file['data_bill']['url_img'], {'x_offset': 260, 'y_offset': 15})
            sheet.merge_range('D2:E4', 'BILL: '+param_export_file['data_bill']['invoice_number'], red_format)
        except:
            pass

        # ///////////////////////////////////////  BILL /////////////////////////////////////////////
        row = 5
        col = 1
        for h in headers:
            sheet.write(row, col, h, grey_dark_bg_bold)
            col += 1
        row = 6
        col = 1

        col_max = len(col_name)
        for trr in queryset:
            for c in col_name:
                if type(trr[c]) == datetime:
                    try:
                        trr[c] = param_export_file['tz'].normalize(trr[c]).strftime('%Y-%m-%d %H:%M:%S')
                    except:
                        trr[c] = str(trr[c])
                if row % 2 != 0:
                    sheet.write(row, col, trr[c], yellow_bg)
                else:
                    sheet.write(row, col, trr[c], grey_bg)

                col += 1
            row += 1
            col = 1
            data = read_and_flush()
            yield data

        # /////////////////////////////////////////////// TOTAL ////////////////////////////////////////
        row = row+1
        sheet.write(row, col_max-3, 'Total Minutes', grey_bg)
        sheet.write(row, col_max-2, total['minutes'], grey_bg)
        sheet.write(row, col_max-1, 'Total', grey_bg)
        sheet.write(row, col_max, total['total_euro'], money_grey_bg)
        row = row+1
        sheet.write(row, col_max-1, 'IVA ' + str(param_export_file['data_bill']['VAT']) + '%', grey_bg)
        iva = float(total['total_euro']) * float(param_export_file['data_bill']['VAT'])/100
        sheet.write(row, col_max, iva, money_grey_bg)
        row = row+1
        sheet.write(row, col_max-1, 'Total + IVA', money_grey_bg)
        total_iva = float(total['total_euro']) * (1 + float(param_export_file['data_bill']['VAT'])/100)
        sheet.write(row, col_max, total_iva, money_grey_bg)

        # /////////////////////////////////////////////  FOOTER DATA ////////////////////////////////////////
        row = row + 3
        coordinate = 'B'+str(row)+':E'+str(row)
        period = 'Issue Date: ' + str(param_export_file['data_bill']['issue_date']) + '      Due Date: '+str(param_export_file['data_bill']['due_date'])
        sheet.merge_range(coordinate, period, merge_format_border)
        row = row + 2
        coordinate = 'B'+str(row)+':E'+str(row)
        period = 'Period From: ' + str(param_export_file['data_bill']['from_date']) + '      Period To: '+str(param_export_file['data_bill']['to_date'])
        sheet.merge_range(coordinate, period, merge_format_border)
        row = row + 2

        coordinate = 'B'+str(row)+':E'+str(row)
        period = 'Payment info for client: ' + str(param_export_file['data_bill']['payment'])
        sheet.merge_range(coordinate, period, merge_format)

        row = row-1

        sheet.write(row+1, 1, 'Bank Name:', flesh_color_bg_bold)
        sheet.write(row+2, 1, 'Account:', flesh_color_bg_bold)
        sheet.write(row+3, 1, 'IBAN:', flesh_color_bg_bold)
        sheet.write(row+4, 1, 'SWIFT:', flesh_color_bg_bold)

        sheet.write(row+1, 2, param_export_file['data_bill']['bank_name'], flesh_color_bg)
        sheet.write(row+2, 2, param_export_file['data_bill']['account'], flesh_color_bg)
        sheet.write(row+3, 2, param_export_file['data_bill']['IBAN'], flesh_color_bg)
        sheet.write(row+4, 2, param_export_file['data_bill']['SWIFT'], flesh_color_bg)

        sheet.write(row+1, 3, 'Company:', flesh_color_bg_bold)
        sheet.write(row+2, 3, 'CIF:', flesh_color_bg_bold)
        sheet.write(row+3, 3, 'Adress:', flesh_color_bg_bold)
        sheet.write(row+4, 3, 'Email:', flesh_color_bg_bold)
        sheet.write(row+5, 3, 'Phone:', flesh_color_bg_bold)

        sheet.write(row+1, 4, param_export_file['data_bill']['company'], flesh_color_bg)
        sheet.write(row+2, 4, param_export_file['data_bill']['company_number'], flesh_color_bg)
        sheet.write(row+3, 4, param_export_file['data_bill']['company_address'], flesh_color_bg)
        sheet.write(row+4, 4, param_export_file['data_bill']['company_email'], flesh_color_bg)
        sheet.write(row+5, 4, param_export_file['data_bill']['company_phone'], flesh_color_bg)

        book.close()

        data = read_and_flush()
        yield data

    if 'format' not in param_export_file:
        param_export_file['format'] = 'bill_pdf'

    if 'filename' not in param_export_file:
        param_export_file['filename'] = 'income_provider.pdf'

    if 'multisheet' in param_export_file:
        multisheet = param_export_file['multisheet']
    else:
        multisheet = False

    if param_export_file['format'] == 'excel':
        if multisheet is False:
            param_export_file['filename'] = param_export_file['filename']+'_' + str(datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))+'.xlsx'

            queryset = qs
            headers = param_export_file['headers']
            col_name = param_export_file['col_name']
            filename = param_export_file['filename']

            if type(queryset) is not list:
                queryset = queryset.iterator()
            try:
                output = StringIO.StringIO()
                filename = param_export_file['STATIC_URL'] + '/excel_csv/' + param_export_file['filename']
                book = Workbook(output, {'constant_memory': True})
                sheet = book.add_worksheet('test')
                bold = book.add_format({'bold': 1})
                col = 0
                row = 0
                for h in headers:
                        sheet.write(row, col, h, bold)
                        col += 1
                row = 1
                col = 0
                for trr in queryset:
                    for c in col_name:
                        if c in trr:
                            if type(trr[c]) == datetime:
                                try:
                                    trr[c] = param_export_file['tz'].normalize(trr[c]).strftime('%Y-%m-%d %H:%M:%S')
                                except:
                                    trr[c] = str(trr[c])

                            if type(trr[c]) is tuple:
                                sheet.write(row, col, trr[c][0], book.add_format(trr[c][1]))
                            else:
                                sheet.write(row, col, trr[c])
                        col += 1
                    row += 1
                    col = 0
                book.close()

            except Exception:
                swslog('error', 'Error in create Excel: ', traceback.format_exc())
            output.seek(0)
            response = HttpResponse(output.read(), content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
            response['Content-Disposition'] = "attachment; filename="+param_export_file['filename']
        else:
            param_export_file['filename'] = param_export_file['filename']+'_' + str(datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))+'.xlsx'

            queryset = param_export_file['queryset']
            headers = param_export_file['headers']
            col_name = param_export_file['col_name']
            filename = param_export_file['filename']
            validation = param_export_file['validation']

            try:
                output = StringIO.StringIO()
                filename = param_export_file['STATIC_URL']+'/excel_csv/' + param_export_file['filename']
                book = Workbook(filename, {'constant_memory': True})

                for i in range(0, len(headers)):
                    sheet = book.add_worksheet(param_export_file['sheet'][i])
                    sheet.set_column('A:A', 30)
                    sheet.set_column('B:B', 30)

                    bold = book.add_format({'bold': 1})
                    col = 0
                    row = 0
                    for h in headers[i]:
                            sheet.write(row, col, h, bold)
                            col += 1
                    row = 1
                    col = 0

                    for trr in queryset[i]:
                        for c in col_name[i]:
                            if c in trr:
                                if type(trr[c]) == datetime:
                                    try:
                                        trr[c] = param_export_file['tz'].normalize(trr[c]).strftime('%Y-%m-%d %H:%M:%S')
                                    except:
                                        trr[c] = str(trr[c])

                                sheet.write(row, col, trr[c])
                            col += 1
                        row += 1
                        col = 0
                #sheet.data_validation('$B$2:$B$10000', {'validate': 'list','source': '=Destinations!$A$2:$A$10000'})
                sheet.data_validation(validation[0], validation[1])
                book.close()
            except:
                swslog('error', 'Error in create Excel: ', traceback.format_exc())

            output.seek(0)
            response = HttpResponse(output.read(), content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
            response['Content-Disposition'] = "attachment; filename="+param_export_file['filename']
        return response

    # CSV case
    elif param_export_file['format'] == 'csv':
        param_export_file['filename'] = param_export_file['filename']+'_'+str(datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))+'.csv'
        try:
            col_title = param_export_file.get('headers', None)
            col_name = param_export_file['col_name']
        except:
            try:
                col_name = param_export_file['col_name']
            except:
                col_name = None
        csv_writer = csv.writer(Echo(), dialect='excel', quoting=csv.QUOTE_MINIMAL, delimiter='|')

        response = StreamingHttpResponse((csv_writer.writerow(row) for row in get_csv_query(param_export_file['request_data'], qs, param_export_file['STATIC_URL'] + '/excel_csv/' + param_export_file['filename'], col_name, col_title, tz=param_export_file['tz'])), content_type='text/csv')
        response['Content-Disposition'] = 'attachment; filename=' + param_export_file['filename']
        return response

    elif param_export_file['format'] == 'bill':

        param_export_file['filename'] = param_export_file['filename']+'_' + str(datetime.now().strftime("%Y-%m-%d_%H-%M-%S"))+'.xlsx'
        filename = param_export_file['filename']
        output = StringIO.StringIO()

        book = Workbook(output)
        sheet = book.add_worksheet('data')

        book.set_properties({
            'title':    param_export_file['filename'],
            'subject':  'Bill',
            'author':   param_export_file['autor'],
            'manager':  'StoneWorkSolutions',
            'company':  'StoneWorkSolutions',
            'category': 'Bill and risk',
            'keywords': 'Bill',
            'comments': 'Created in portal international'})

        response = HttpResponse(writeBillInStream(), content_type="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet")
        response['Content-Disposition'] = "attachment; filename=" + filename
        return response
    elif param_export_file['format'] == 'bill_pdf':
        render_to_pdf_result = render_to_pdf('tags/pdf_format.html', context_instance=param_export_file, header_template=None, footer_template='tags/tag_footer.html', filename=param_export_file['filename'])
        if param_export_file['filename']:
            if not os.path.isfile(param_export_file['filename']):
                raise Exception('File {0} couldn\'t be created'.format(param_export_file['filename']))
        return render_to_pdf_result


def get_csv_query(request, queryset, filename, col_order=None, col_title=None, in_stream=False, tz=None):
    try:
        if type(queryset) is not list:
            # Get field names
            field_names = []
            if col_order:
                field_names = col_order
            else:
                for f in queryset._fields:
                    field_names.append(unicode(getFieldVerboseName(queryset, f)))
                for f in getattr(queryset, 'aggregate_names', []):
                    field_names.append(f)

            # Get field types
            field_types = []
            for f in queryset._fields:
                field_types.append(unicode(getFieldType(queryset, f)))
            for f in getattr(queryset, 'aggregate_names', []):
                try:
                    field_types.append(unicode(getFieldType(queryset, f)))
                except:
                    field_types.append(u'FloatField')
            cont = 0
            first_it = True
            for q in queryset.iterator():
                if first_it:
                    first_it = False
                    if col_title:
                        yield col_title
                    else:
                        yield field_names
                aux = []
                i = 0
                for f in field_names:

                    if field_types[i] == 'DateTimeField':
                        try:
                            aux.append(pytz.utc.normalize(q[f]).astimezone(tz).strftime('%Y-%m-%d %H:%M:%S'))
                        except:
                            aux.append(q[f])
                    else:
                        aux.append(q[f])
                    i += 1
                for f in getattr(queryset, 'aggregate_names', []):
                    aux.append(q[f])
                cont += 1
                yield aux

        else:
            field_names = []

            if col_order:
                field_names = col_order
            else:
                for k, v in queryset[0].items():
                    field_names.append(unicode(k))

            first = True
            cont = 0
            # v_data.append(q[k])
            for i in range(-1, len(queryset)):
                if first:
                    first = False
                    yield col_title
                else:
                    v_data = []
                    for k in field_names:
                        if k in queryset[i]:
                            v_data.append(queryset[i][k])
                    cont += 1
                    yield v_data

    except Exception, e:
        swslog('error', 'get_csv_query', e)
        yield None


def get_field_types(fields):
    field_types = []
    found = False
    for lf in fields:
        found = False
        if type(lf) == unicode:
            field_types.append('text')
            found = True
        elif type(lf) == long:
            field_types.append('int')
            found = True
        elif type(lf) == int:
            field_types.append('int')
            found = True
        elif type(lf) == bool:
            field_types.append('boolean')
            found = True
        elif type(lf) == float:
            field_types.append('price')
            found = True
        elif type(lf) == Decimal:
            field_types.append('price')
            found = True
        elif type(lf) == date:
            field_types.append('date')
            found = True
        elif type(lf) == datetime:
            field_types.append('date')
            found = True
        if found is not True:
            field_types.append('text')
    return field_types


def listToExport(main_fields, remove_fields=None, seconds=None):
    '''Remove fields which must not be in export_file
    :param main_fields: all fields of the grid
    :type main_fields: list
    :param remove_fields: fields to remove
    :type remove_fields: list
    :param seconds: repeated fields, we only want to remove the second, defaults to []
    :type seconds: list, optional
    :returns: the fields which will appear in the file to export
    :rtype: {list}
    '''
    if not remove_fields:
        remove_fields = []
    if not seconds:
        seconds = []
    for remove_field in remove_fields:
        if remove_field in seconds:
            main_fields.reverse()
        try:
            main_fields.remove(remove_field)
        except ValueError:
            pass
        if remove_field in seconds:
            main_fields.reverse()
    return main_fields



#!/usr/bin/env python
# encoding: utf-8

# from django.template.context_processors import csrf
# from international.views import *  TODOMAS

# @csrf_exempt
# def requestEmail(request):
# 	try:
# 		ms = MailSubscription.objects.get(user=request.user.id)
# 		email = ms.destination_email
# 		period =ms.period
# 	except:
# 		period = "never"	
# 		# Get the mail address if the current user is related to a client and doesnt have subscriptions yet
# 		# try:  TODOMAS
# 		# 	email = Client.objects.get(user=request.user.id).email
# 		# 	if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", email) == None: raise
# 		# except:
# 		email =""
# 	return email,period

# # @user_ok
# # @csrf_exempt
# def save_subscription_settings(request):
# 	usr = User.objects.get(id=request.user.id)
# 	if 'email' in request.POST and 'period' in request.POST:
# 		email = request.POST['email']
# 		period = request.POST['period']
# 		# if there is no addres or is not valid, try to get the address from the client
# 		if email == None or re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", email) == None:

# 			t = Template('KO: not valid email address')
# 			return HttpResponse(t.render(RequestContext(request)))
# 		# Check that the period is vaid
# 		if period in ('day','week','fifteen','month','never'):
# 			try:
# 				ms = MailSubscription.objects.get(user=usr)
# 				ms.delete()
# 			except:
# 				pass
# 			try:
# 				ms	= MailSubscription(period=period,destination_email=email,user=usr,created_by=usr)
# 				ms.save()
# 			except Exception, e:
# 				swslog('error','KO: Could not save the subscription settings pat',e)
# 				t = Template('KO: Could not save the subscription settings pat')				
# 		else:
# 			t = Template('KO: Incorrect parameters')
# 	else:
# 		t = Template('KO: Incorrect parameters')
	
# 	t = Template('OK: Settings updated, message sent')
	
# 	sendmail(request,'subscription_changed',email)
	
# 	return HttpResponse(t.render(RequestContext(request)))
	


# def sendmail(request,code,to):
# 	from_field = settings.DEFAULT_FROM_EMAIL

# 	mails ={
# 		'subscription_changed': ['[MyRealStats] Mail subscription changed', 'Your mail CDR subscription has been successfully updated to the new settings']
# 	}
# 	if code in mails:
# 		status = send_mail(mails[code][0],mails[code][1], from_field,[to], fail_silently=False)

# 	else: 
# 		return 0
		
# 	return status


from os import fdopen, remove
from subprocess import Popen, PIPE, CalledProcessError
from tempfile import mkstemp

# from django.conf import settings
from django.http import HttpResponse
from django.template import loader
from django.utils.encoding import smart_str

# WKHTMLTOPDF_CMD = getattr(settings, 'WKHTMLTOPDF_CMD', 'wkhtmltopdf')

WKHTMLTOPDF_CMD = 'wkhtmltopdf'

class CalledProcessDetailedError(CalledProcessError):
	def __init__(self, returncode, cmd, process_output):
		CalledProcessError.__init__(self, returncode, cmd)
		self.process_output = process_output.lstrip()
	def __str__(self):
		return CalledProcessError.__str__(self) + "\n%s" % self.process_output


def wkhtmltopdf(pages, output='media/reg/test2.pdf', **kwargs):
	"""
	Converts html to PDF using http://code.google.com/p/wkhtmltopdf/.

	pages: List of file paths or URLs of the html to be converted.
	output: Optionsal output file path.
	**kwargs: Passed to wkhtmltopdf via _extra_args() (See
			  https://github.com/antialize/wkhtmltopdf/blob/master/README_WKHTMLTOPDF
			  for acceptable args.)
			  Kwargs is passed through as arguments. e.g.:
				  {'footer_html': 'http://example.com/foot.html'}
			  becomes
				  '--footer-html http://example.com/foot.html'
			  Where there is no value passed, use a blank string. e.g.:
				  {'disable_javascript': ''}
			  becomes:
				  '--disable-javascript '

	example usage:
		wkhtmltopdf(html_path="~/example.html",
					dpi=300,
					orientation="Landscape",
					disable_javascript="")
	"""

	def _extra_args(**kwargs):
		"""Converts kwargs into a string of flags to be passed to wkhtmltopdf."""
		flags = ''
		for k, v in kwargs.items():
			if k!= 'javascript-delay':
				flags += ' --%s %s' % (k.replace('_', '-'), v)
		return flags

	if not isinstance(pages, list):
		pages = [pages]

	kwargs['quiet'] = ''
	# kwargs['javascript-delay'] = 3000
	args = '%s %s %s %s' % (WKHTMLTOPDF_CMD, _extra_args(**kwargs), ' '.join(pages), output or '-')
	process = Popen(args, stdout=PIPE, stderr=PIPE, shell=True)
	stdoutdata, stderrdata = process.communicate()

	if process.returncode != 0:
		# combined_program_output = "stdout: %s\nstderr: %s" % (stdoutdata, stderrdata)
		# raise CalledProcessDetailedError(process.returncode, args, combined_program_output)

		output = stdoutdata
		return output


	if output is None:
		output = stdoutdata

	return output

def render_to_pdf(template_name, context_instance=None, header_template=None, footer_template=None,filename=None, **kwargs):
	"""Renders a html template as a PDF response."""
	page_path = template_to_temp_file(template_name, context_instance)

	tmp_files = []
	if header_template is not None:
		kwargs['header_html'] = template_to_temp_file(header_template, context_instance)
		tmp_files.append(kwargs['header_html'])
	if footer_template is not None:
		kwargs['footer_html'] = template_to_temp_file(footer_template, context_instance)
		tmp_files.append(kwargs['footer_html'])


	content = wkhtmltopdf([page_path],filename, **kwargs)

	for path in tmp_files:
		remove(path)

	response = HttpResponse(content, content_type='application/pdf')
	if filename is not None:
		response['Content-Disposition'] = 'attachment;' + ' filename=%s' %filename
	return response

def template_to_temp_file(template_name, context, request=None):
	"""Renders a template to a temp file, and returns the path of the file."""
	fd, tmppath = mkstemp(suffix='.html')
	f = fdopen(fd, 'wt')
	f.write(smart_str(loader.render_to_string(template_name, context=context, request=request)))
	f.close()
	return tmppath

#wkhtmltopdf()

# #########################################################################################################
# #########################################################################################################
# ############################################### PHANTOMJS ###############################################
# #########################################################################################################
# #########################################################################################################
import subprocess
from django.shortcuts import get_object_or_404, render_to_response

# JS_PATH  = settings.JS_PATH
# FIX: Pero esto que es!!??
JS_PATH  = '/home/msanchez/projects/ooint/static/js/htmlToPdf.js'

# New function to render HTML as PDF
def phantomjs_pdf(template,context_instance):

	# page_path = template_to_temp_file(template, context_instance)

	# command = 'phantomjs {0} {1} /tmp/payment_{2}.pdf'.format(JS_PATH, page_path, context_instance['payment'])
	# # print 'command =',command

	# try:
	# 	subprocess.check_call(command,shell=True)
	# except:
	# 	raise

	filename = None
	filename = 'payment_{0}.pdf'.format(context_instance['payment'])

	response = HttpResponse(filename, content_type='application/pdf')

	if filename is not None:
		response['Content-Disposition'] = 'attachment;' + ' filename=%s' %filename
	return response



import ujson
import inspect
from messages import sendMessages
from datetime import datetime  # timedelta#,date
from collections import OrderedDict as SortedDict
from dateutil import parser
import copy
from sws_tags.sws_utils.messages import *
from sws_tags.sws_utils.cube_utils import *
from sws_tags.sws_utils.date import *
from sws_tags.sws_utils.__init__ import *


def simboloYvalorTZ(date):
    ##extrae de objeto del tipo datetime el valor y signo de la franja horaria.
    date_str = str(date)

    try:
        if (date_str.find('+') > -1):
            tz = date_str[date_str.find('+')+1:]
            tz = int(tz[:tz.find(':')])
            simbolo = '+'
        else:
            tz = date_str[date_str.find(' ')+1:]
            tz = tz[tz.find('-')+1:]
            tz = int(tz[:tz.find(':')])
            simbolo = '-'
    except:
        simbolo = '+'
        tz = 0
    return simbolo, tz


def checkIfTimeToTz(part_where,session_tz,system_tz):
    tiempo={'Time':False,'Date':False}
    for i in part_where:
        if i[0]in tiempo:
            tiempo[i[0]]=i
            dim_time=i
    
    dates=changeTimeByTz(part_where[dim_time],session_tz,system_tz)

    part_where_aux_fechas = {}
    for i in part_where:
        if i == dim_time:
            part_where_aux_fechas[i]=dates
        else:
            part_where_aux_fechas[i]=part_where[i]

    part_where=part_where_aux_fechas
    
    return part_where

def checkIfTimeToTzNew(part_where, session_tz, system_tz, format='%Y%m%d%H%M%S'):
    tiempo = {'Time': False, 'Date': False}
    for i in part_where:
        if i[0] in tiempo:
            tiempo[i[0]] = i
            dim_time = i
    dates = changeTimeByTzNew(part_where[dim_time], session_tz, system_tz, format)
    part_where_aux_fechas = {}
    for i in part_where:
        if i == dim_time:
            part_where_aux_fechas[i] = dates
        else:
            part_where_aux_fechas[i] = part_where[i]
    part_where = part_where_aux_fechas
    return part_where


def modifyNameBddCubes(name_bbdd, request_data):    
    if name_bbdd[0]=='ing' or name_bbdd[0]=='sultan':
        yearFrom = request_data['from_calldate'].strftime("%Y-%m-%d %H:%M:%S")[0:4]
        yearTo = request_data['to_calldate'].strftime("%Y-%m-%d %H:%M:%S")[0:4]
        monthFrom = request_data['from_calldate'].strftime("%Y-%m-%d %H:%M:%S")[5:7]
        # monthTo = request_data['to_calldate'].strftime("%Y-%m-%d %H:%M:%S")[5:7]
        if (yearTo==yearFrom):
            year=yearFrom
        else:
            if int(monthFrom)>=10 :
                year=yearTo
            else:
                year=0
        for i in range(len(name_bbdd)):
            name_bbdd[i]=name_bbdd[i]+str(year)
    return name_bbdd

def reorderColCube(col_cube,data_order):
    col_cube_aux=[]
    if col_cube:
        if data_order in col_cube:
            col_cube.remove(data_order)
            col_cube=sorted(list(set(col_cube)))
            col_cube_aux.append(data_order)
            for i in col_cube:
                col_cube_aux.append(i)

            return col_cube_aux
        else:
            return col_cube
    return col_cube


def add_MDX_redis(ide,value,r,seconds):            
            
    return r.set(ide+'_MDXS',value,seconds)
    
def get_MDXS_redis(ide,r):

    value = eval(r.get(ide+'_MDXS'))
    
    return value
def is_MDXS_Added_redis(ide,r):                                                                                    
    return not (r.get(ide+'_MDXS') == None)

def segundosHastaY5oEnPunto(minute,seconds):
    if len(str(minute))==1:
        aux=str(minute).zfill(2)
    else:
        aux=str(minute)

    digit1=int(aux[1])
    digit0=int(aux[0])

    if digit1<5:
        digit1=5
    else:
        digit1=0
        digit0=digit0+1

    return ((int(str(digit0)+str(digit1))-minute)*60)-seconds


def dateCubeRouter(func_name):
    ''' Selects wich cube will be queried depending on the date range
    Will raise an error if the schema change date is within th date range
    or Sales Rep is used as filter o group column on the old schema'''
    def decorate(func):
        def wrapper(*args, **kwargs):
            request_data = args[5]
            from_date = request_data['from_Date']
            to_date = request_data['to_Date']

            # The query contains a data period that involves two schemas
            if from_date < request_data['var_settings']['CUBE_CHANGE_DATE'] and to_date > request_data['var_settings']['CUBE_CHANGE_DATE']:
                sendMessages(
                    request_data['var_settings']['USER_MESSAGE_CHANNEL'],
                    kwargs['redis'],
                    message='Date range can not contain two different schemas (schema changed on {0})'.format(request_data['var_settings']['CUBE_CHANGE_DATE']),
                    level='warning',
                    time=5000,
                    rsyslog=False
                )
                # Return depending on the decorated function
                if func_name == 'requestCube':
                    return ujson.dumps({"records":0,"total":0.0,"rows":[],"page":1})
                elif func_name == 'requestCubeFilters':
                    return ujson.dumps([])
                elif func_name == 'requestCubeHighcharts':
                    return {'v_name': ['wrong date period', 'wrong date period'], 'v_data': [], 'v_yAxis': [0, 1], 'v_type': ['line'], 'v_xAxis': ['2017-01-01']}

            # Route the connection to the legacy database
            elif to_date < request_data['var_settings']['CUBE_CHANGE_DATE']:
                # Can be string or iterable
                if hasattr(args[1], '__iter__'):
                    args[1][0] = '{0}_29'.format(args[1][0])
                else:
                    args[1] = '{0}_29'.format(args[1])
                if 'cube_config' in request_data:
                    # The deep copy is necessary because it comes from the session and would be modified system wide
                    args = copy.deepcopy(args)
                    request_data = args[5]
                    request_data['cube_config']['cube_db'] = '{0}_29'.format(request_data['cube_config']['cube_db'])

                # Send a message indicating that the Salesrep filter will have no effect
                if request_data.get('Salesrep', 'NULL') != 'NULL':
                    sendMessages(
                        request_data['var_settings']['USER_MESSAGE_CHANNEL'],
                        kwargs['redis'],
                        message='Sales Rep filter will have no effect on this period',
                        level='warning',
                        time=5000,
                        rsyslog=False
                    )

                # Send a message indicating that can't group by Salesrep on this date period
                for field in ('group_by', 'and_by', 'final_by'):
                    if request_data.get(field, 'NULL') == 'Salesrep':
                        sendMessages(
                            request_data['var_settings']['USER_MESSAGE_CHANNEL'],
                            kwargs['redis'],
                            message='Sales Rep can not be used as group column on this period',
                            level='warning',
                            time=5000,
                            rsyslog=False
                        )
                        break

            # Connect to the default cube
            elif from_date > request_data['var_settings']['CUBE_CHANGE_DATE']:
                pass

            # Execute the function
            res = func(*args, **kwargs)
            return res
        return wrapper
    return decorate


@dateCubeRouter('requestCube')
def requestCube(ip_connect, name_bbdd, name_cube, col_cube, med_cube, request_data, ifDate, dimension_filters, database_cube_dict, total_data=False, where_tupla=('Date', 'Id', 'Id'), request_date_previous=False, complete_name=False, format_cube=False, n_c=[], time_zone="", redis=""):

    if 'cube_config' in request_data:
        cube_config = request_data['cube_config']
        ip_connect = cube_config['cube_ip']
        name_bbdd = cube_config['cube_db']
        name_cube = request_data['cube_name']
        limit_sim_exe = cube_config['limit_sim_exe']
        limit_time = cube_config['limit_time']

    cube = CUBE(limit_sim_exe=limit_sim_exe, limit_time=limit_time, version=request_data['var_settings']['VERSION'])
    mdx_class = MDX()
    cube.connect(ip_connect, name_bbdd, n_c, redis)
    exclude_rows = {}
    where = {}
    data_order = ''
    part_order = {}
    mdx = ''
    part_where = {}
    part_where = filterWhere(dimension_filters, request_data, database_cube_dict)
    for i in part_where:
        where[i] = part_where[i]

    if ifDate:
        if not request_date_previous:
            where[where_tupla] = (formatDateCube(request_data['from_Date']), formatDateCube(request_data['to_Date']))
        else:
            where[where_tupla] = (formatDateCube(request_data['from_Date_previous']), formatDateCube(request_data['to_Date_previous']))
    sord = request_data['sord'].upper()
    sord = 'B'+sord
    for i in col_cube:
        if i[0] == 'Date':
            dato = i[1]
        else:
            dato = i[0]
        if request_data['sidx'] == dato:
            data_order = i

    for i in med_cube:
        if request_data['sidx'] == i[1]:
            data_order = i
    col_cube = namesColCube(col_cube)
    data_order = namesColCube(data_order)
    part_order[data_order] = sord
    if ifDate:
        where = checkIfTimeToTz(where, request_data['django_timezone'], request_data['system_timezone'])
    col_cube = reorderColCube(col_cube, data_order)
    key = name_bbdd+'MDXS_'+str(col_cube)+str(exclude_rows)+str(med_cube)+str(name_cube)+str(where)+str(part_order)+str(name_bbdd)+str(ip_connect)+str(format_cube)
    minute = datetime.now().minute
    second = datetime.now().second
    seconds = segundosHastaY5oEnPunto(minute, second)
    if (is_MDXS_Added_redis(name_bbdd, redis)):
        MDXS = get_MDXS_redis(name_bbdd, redis)
    else:
        MDXS = {}
    if not (key in MDXS.keys()):
        mdx = mdx_class.mdx(cube, part_rows=col_cube, exclude_rows=exclude_rows, part_columns=med_cube, part_from=name_cube, part_where=where, part_order=part_order)  # ASC or DESC  ('measure', 'calls')
        res = cube.launch_query(mdx)
        grid = Format()
        if total_data:
            json_data = grid.grid(cube, res, database_cube_dict, False, request_data['row_num'], request_data['page'], total_data, complete_name, format_cube, col_cube)
            mdx_total = 'select ([Client].[Client]) on rows,'+mdx_class.without_rows(mdx)
            res = cube.launch_query(mdx_total)
            json_data_footer = grid.grid(cube, res, database_cube_dict, False, request_data['row_num'], request_data['page'], total_data, complete_name, format_cube)
            json_data = json.loads(json_data)
            if json.loads(json_data_footer)['rows']:
                json_data['footerData'] = json.loads(json_data_footer)['rows'][0]
                try:
                    json_data['footerData'][request_data['group_by']] = 'ALL'
                except:
                    pass
                try:
                    json_data['footerData'][request_data['and_by']] = 'ALL'
                except:
                    pass
            json_data = json.dumps(json_data)
        else:
            json_data = grid.grid(cube, res, database_cube_dict, False, request_data['row_num'], request_data['page'], total_data, complete_name, format_cube, col_cube)
        if data_order:
            if data_order in col_cube:
                if 'ASC' in sord:
                    json_data = ujson.loads(json_data)
                    json_data = reverseJson(json_data)
                    json_data = ujson.dumps(json_data)
        MDXS[key] = ujson.loads(json_data)
        add_MDX_redis(name_bbdd, MDXS, redis, seconds)
    else:
        json_data = MDXS[key]
        json_data = ujson.dumps(json_data)
    return json_data


def reverseJson(json_data):
    rows = json_data['rows']
    rows.reverse()
    json_data['rows'] = rows
    return json_data


def reorderJson(json_data, col_cube, data_order):
    dim1 = col_cube[0][0]+'*'+col_cube[0][1]
    dim2 = col_cube[1][0]+'*'+col_cube[1][1]

    for row in json_data['rows']:
        value_dim1 = row[dim1]
        value_dim2 = row[dim2]
        row[dim1] = value_dim2
        row[dim2] = value_dim1

    if len(col_cube) > 2:
        if data_order == col_cube[2]:
            dim1 = col_cube[1][0]+'*'+col_cube[1][1]
            dim2 = col_cube[2][0]+'*'+col_cube[2][1]

            for row in json_data['rows']:
                value_dim1 = row[dim1]
                value_dim2 = row[dim2]
                row[dim1] = value_dim2
                row[dim2] = value_dim1
    return json_data

@dateCubeRouter('requestCubeHighcharts')
def requestCubeHighcharts(ip_connect, name_bbdd, name_cube, col_cube, med_cube, request_data, ifDate, dimension_filters, database_cube_dict, total_data=True, where_tupla=('Date', 'Id', 'Id'), request_date_previous = False,  format_cube=False, n_c=[], time_zone="", redis=""):
    if 'cube_config' in request_data:
        cube_config = request_data['cube_config']
        ip_connect = cube_config['cube_ip']
        name_bbdd = cube_config['cube_db']
        name_cube = request_data['cube_name']
        limit_sim_exe = cube_config['limit_sim_exe']
        limit_time = cube_config['limit_time']


    cube = CUBE(limit_sim_exe=limit_sim_exe, limit_time=limit_time, version=request_data['var_settings']['VERSION'])
    mdx_class = MDX()
    cube.connect(ip_connect, name_bbdd, n_c, redis)
    exclude_rows = {}
    where = {}
    part_where = filterWhere(dimension_filters, request_data, database_cube_dict)
    for i in part_where:
        where[i] = part_where[i]

    if ifDate:
        if not request_date_previous:
            where[where_tupla] = (formatDateCube(request_data['from_Date']), formatDateCube(request_data['to_Date']))
        else:
            where[where_tupla] = (formatDateCube(request_data['from_Date_previous']), formatDateCube(request_data['to_Date_previous']))

    new_col_cube = []
    for i in col_cube:
        new_col_cube.append((i[0], i[1], i[2]))
    col_cube = new_col_cube
    request_data['int_all_time'] = 0
    where = checkIfTimeToTz(where, request_data['django_timezone'], request_data['system_timezone'])
    key = name_bbdd+'MDXS_'+str(col_cube)+str(exclude_rows)+str(med_cube)+str(name_cube)+str(where)+str(name_bbdd)+str(ip_connect)+str(request_data['button'])
    minute = datetime.now().minute
    second = datetime.now().second
    seconds = segundosHastaY5oEnPunto(minute, second)

    if (is_MDXS_Added_redis(name_bbdd, redis)):
        MDXS = get_MDXS_redis(name_bbdd, redis)
    else:
        MDXS = {}

    if not (key in MDXS.keys()):
        mdx = mdx_class.mdx(cube, part_rows=col_cube, exclude_rows=exclude_rows, part_columns=med_cube, part_from=name_cube, part_where=where)  # ASC or DESC  ('measure', 'calls')
        res = cube.launch_query(mdx)
        high = Format()
        types = request_data['representation_type']
        if request_data['button'] == 'ALL':
            json_data = high.highchart(cube, res, types, dimension_v_name='Client', format_cube=format_cube)
        else:
            json_data = high.highchart(cube, res, types, dimension_v_name=request_data['button'], exclude=exclude_rows, format_cube=format_cube)

        MDXS[key] = json_data
        add_MDX_redis(name_bbdd, MDXS, redis, seconds)
    else:
        json_data = MDXS[key]
    return json_data

@dateCubeRouter('requestCubeFilters')
def requestCubeFilters(ip_connect, name_bbdd, name_cube, col_cube, med_cube, request_data, ifDate, dimension_filters, database_cube_dict=False, where_tupla=('Date', 'Id', 'Id'), n_c=[], time_zone="", redis=""):
    # name_carrier=''
    if 'cube_config' in request_data:
        cube_config = request_data['cube_config']
        ip_connect = cube_config['cube_ip']
        name_bbdd = cube_config['cube_db']
        name_cube = request_data['cube_name']
        limit_sim_exe = cube_config['limit_sim_exe']
        limit_time = cube_config['limit_time']
    cube = CUBE(limit_sim_exe=limit_sim_exe, limit_time=limit_time, version=request_data['var_settings']['VERSION'])
    mdx_class = MDX()
    cube.connect(ip_connect, name_bbdd, n_c, redis)
    where = filterWhere(dimension_filters, request_data, database_cube_dict)
    exclude_rows = {}
    if request_data[request_data['types']] != 'NULL':
        if request_data['types'] == 'Hangupcode2':
            key = ('Hangupcode2', 'Long Description', 'Long Description')

        elif request_data['types'] == 'Hangupcode':
            key = ('Hangupcode', 'Description', 'Description')
        else:
            key = (str(request_data['types']), 'Id', 'Id')
        col_cube.append(key)
        where_aux = {}
        for i in where:
            if i != key:
                where_aux[i] = where[i]
        where = where_aux
    else:
        exclude_rows = {}
    if ifDate:
        where[('Date', 'Id', 'Id')] = (formatDateCube(request_data['from_Date']), formatDateCube(request_data['to_Date']))
        where = checkIfTimeToTz(where, request_data['django_timezone'], request_data['system_timezone'])
    key = name_bbdd + 'MDXS_' + str(col_cube) + str(exclude_rows) + str(med_cube) + str(name_cube) + str(where) + str(name_bbdd) + str(ip_connect) + str(dimension_filters)

    minute = datetime.now().minute
    second = datetime.now().second
    seconds = segundosHastaY5oEnPunto(minute, second)
    MDXS = {}
    if not (key in MDXS.keys()):
        mdx = mdx_class.mdx(cube, part_rows=col_cube, exclude_rows=exclude_rows, part_columns=med_cube, part_from=name_cube, part_where=where)
        grid = Format()
        dimension_filter = filterDimensionCube(dimension_filters, request_data['types'])
        if dimension_filter[0][0].find('_') > 0:  # especial case two atributes same dimension, delete charts after '_'
            dimension_filter_new = []
            index = dimension_filter[0][0].find('_')
            dimension_filter_new.append((dimension_filter[0][0][0:index], dimension_filter[0][1], dimension_filter[0][2]))
            dimension_filter = dimension_filter_new
        if dimension_filter[0][0] == 'Hangupcode':
            dimension_filter = [('Hangupcode', 'Description', 'Description')]
            filters_cube = grid.filters(mdx, cube, database_cube_dict, dimension_filter, exclude_rows, 'None')
        elif dimension_filter[0][0] == 'Hangupcode2':
            dimension_filter = [('Hangupcode2', 'Long Description', 'Long Description')]
            filters_cube = grid.filters(mdx, cube, database_cube_dict, dimension_filter, exclude_rows, 'None')
        else:
            filters_cube = grid.filters(mdx, cube, database_cube_dict, dimension_filter, exclude_rows)
        MDXS[key] = filters_cube
        add_MDX_redis(name_bbdd, MDXS, redis, seconds)
    else:
        filters_cube = MDXS[key]
    json_data = json_encode(filters_cube)
    return json_data


def filterColCube(col_cube,col_group_by,col_and_by='',col_final_by=''):
    # exclusivas = ['Month Field','Week Field','Day Complete']
    col_cube_new = []
    if (col_group_by!='ALL')&(col_group_by!='All'):
        if (col_group_by!=''):
            added=False
            for c in col_cube:
                if (c[0]==col_group_by or c[2]==col_group_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))
                    added=True   

    if (col_and_by!='ALL')&(col_and_by!='All'):
        if (col_and_by!='' and col_and_by!= col_group_by):
            added=False
            for c in col_cube:
                if (c[0]==col_and_by or c[2]==col_and_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))
                    added=True

    if (col_final_by!='ALL')&(col_final_by!='All'):
        if (col_final_by!='' and col_final_by!= col_and_by and col_final_by!= col_group_by):
            added=False
            for c in col_cube:
                if (c[0]==col_final_by or c[2]==col_final_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))
                    added=True
    return col_cube_new


def namesColCube(col_cube_new):

    if type(col_cube_new) == list:
        col_cube=[]
        for c in col_cube_new:
            index = c[0].find('_')
            if index>0:
                col_cube.append((c[0][0:index],c[1],c[2]))
            else:
                col_cube.append((c[0],c[1],c[2]))
        return col_cube  
    else:
        if len(col_cube_new)>0:
            try:
                index = col_cube_new[0].find('_')
                if index>0:
                    new_name = col_cube_new[0][0:index]
                    col_cube_new=(new_name,col_cube_new[1],col_cube_new[2])
            except Exception, e:
                swslog('error','Index out range',e)
        return col_cube_new


def adjustementData(dict_higcharts,value_adjustement):
    list_index = []
    i=0

    for data in dict_higcharts['v_name']:
        for name in value_adjustement:
            if data.find(name[0]) > 0:
                list_index.append((data,i,name[1]))
                i = i+1

    list_data = []
    list_aux = []

    j = 0
    for data in dict_higcharts['v_data']:
        for i in range(0, len(data)):
            try:
                list_aux.append(data[i]*list_index[j][2])
            except:
                list_aux.append(data[i])
        j = j+1
        list_data.append(list_aux)
        list_aux = []
    dict_higcharts['v_data'] = list_data

    return dict_higcharts


#return tuple with column and id of the dimemsion selected
def filterDimensionCube(dimension_filters, dimension):
    new_dimension_filters = []
    for d in dimension_filters:
        if d[0] == dimension:
            new_dimension_filters.append(d[0:3])
    return new_dimension_filters

def filterWhere(filters,request_data,database_cube_dict): 
    where={}
    for f in filters:
        if f[0]!='calldate':
            if f[0] in request_data:
                if  request_data[f[0]]!='NULL':
                    if f[0]=='Hangupcode':
                        where[(f[0],'Description','Description')]=request_data[f[0]]
                    elif f[0]=='Hangupcode2':
                        where[(f[0],'Long Description','Long Description')]=request_data[f[0]]
                    else:    
                        where[(f[0],'Id','Id')]=request_data[f[0]]
    # Special Case Sales Rep
    if 'sales_rep_id' in request_data:
        where[('Salesrep','Id','Id')]=str(request_data['sales_rep_id'])

    if request_data['param_exclude'] == '1': 
        where[('Hangupcode','Id','Id')]=('-',['34','41'])
    return where


def filterDataCube(filters, request, database_cube_dict):
    where = {}
    for f in filters:
        if f[0] != 'calldate':
            if f[0] in request.GET:
                if request.GET[f[0]] != 'NULL':
                    where[(f[0], 'id')] = request.GET[f[0]]
    return where


def filterMed(med_cube, select_med):

    if select_med.lower() == 'asr':
        med_cube=[('Measures','ASR'),('Measures','Attempts')]
        return med_cube
    elif select_med.lower() == 'acd' :
        med_cube=[('Measures','ACD'),('Measures','ASR'),('Measures','Attempts')]
        return med_cube
    else:
        return med_cube


def atributeAdjustement(json_data,col_cube,request_data):
    if len(json_data)>70:
        json_data = json_data.replace('Date*','')
        
        #especial case when the query have two atributtes of the same dimension
        if request_data['group_by'].find('_')>0 :
            group_by_plus= request_data['group_by'].replace('_','*')
            json_data = json_data.replace(group_by_plus,request_data['group_by'])

        if 'and_by' in request_data:
            try:
                if request_data['and_by'].find('_')>0:
                    and_by_plus = request_data['and_by'].replace('_','*')
                    json_data = json_data.replace(and_by_plus,request_data['and_by'])
            except Exception, e:
                swslog('error', 'non request_data[and_by]',e)

        for name in col_cube:
            json_data = json_data.replace('*'+name[1]+'"','"')
   
    return json_data


def obtainMargin(json_data):
    json_data_dict = ujson.loads(json_data)
    
    for data in json_data_dict['rows']:
        if float(data['Total Income']) != 0:
            # benefit = float(data['Total Income']) - float(data['Total Cost'])
            # margin = benefit / float(data['Total Income']) * 100
            dif = random.randrange(0,9)
            data['Margin'] = dif
        else:
            data['Margin'] = 0
    json_data = json.dumps(json_data_dict)

    return json_data


def orderDataCube(request,json_data):
    json_data_dict = ujson.loads(json_data)
    name_order = request['sidx']
    if request['sord'] == 'asc':
        rev = False
    else:
        rev = True
    sorted_json = sorted(json_data_dict['rows'], key=lambda d: d[name_order],reverse = rev)
    json_data_dict['rows'] = sorted_json
    json_data_dict = json.dumps(json_data_dict)

    return json_data_dict


def accuratelyAdjustment(json_data,med_cube_prec,dict_type_data = False,request_data=False,format_excel= False):

    # request = ThreadLocal.get_current_request()

    if len(med_cube_prec[0])==3:
        ind = 0
    else:
        ind = 1

    json_data_dict = json.loads(json_data,object_pairs_hook=SortedDict)

    new_list_data = []
    for i in range(len(json_data_dict['rows'])):
        new_dict = SortedDict()
        for k,v in json_data_dict['rows'][i].items():
            for m in med_cube_prec:
                if m[ind] == k:

                    if type(v) not in (str, unicode):
                        v = str(v)

                    is_ip = False
                    ini_v = v.find('.')
                    sub_v= v[ini_v+1:]
                    
                    if sub_v.find('.')>=0:
                        is_ip=True


                    extension = False
                    try:
                        int(v[ini_v+1:]) 
                    except:
                        extension = True

                    if v.find('.')>=1 and v != '0.0.0.0' and is_ip==False and extension == False:

                        index = v.find('.');
                        if m[ind+1] != 0:
                            v = v[0:index+1+m[ind+1]]
                        else:
                            v = v[0:index]
                        
                        if format_excel == True:
                            v = float(v)
                        else :
                            v = v.replace('.',',')
                            if dict_type_data:
                                if k in dict_type_data and dict_type_data[k]:

                                    try:    
                                        float(v)             
                                        v = numberFormatter(v)
                                    except Exception, e:
                                        pass
                                        # swslog('error','Number formater not float:'+str(type(v))+str(v),e)
                            try:
                                v = v + m[ind+2].decode('utf-8')
                            except Exception, e:
                                swslog('error','Add type data for measure cube',e)


            new_dict[k] = v
        new_list_data.append(new_dict)
    json_data_dict['rows'] = new_list_data

    try:

        new_dict_footer={}
        if 'footerData' in json_data_dict:
            for k,v in json_data_dict['footerData'].items():
                for m in med_cube_prec:
                    if m[ind] == k:
                        v = str(v)
                        v = v.replace(',','.')
                        if v.find('.')>=1 and v != '0.0.0.0':
                            index = v.find('.');
                            if m[ind+1] != 0:
                                v = v[0:index+1+m[ind+1]]
                            else:
                                v = v[0:index]

                            if dict_type_data:
                                if dict_type_data[k]:
                                    try:
                                        float(v)             
                                        v = numberFormatter(v)
                                    except Exception, e:
                                        swslog('error','Number formater not float:'+str(type(v))+str(v),e)
                        try:
                            v = v + m[ind+2]
                        except Exception, e:
                            # logger.error('ERROR: in add type data for measure cube')
                            # logger.error('Exception: {0}'.format(e))
                            swslog('error','Add type data for measure cube',e)
                    # new_dict[k] = v
        
                new_dict_footer[k] = v
            json_data_dict['footerData'] = new_dict_footer
    except Exception,e:
        swslog('info','json without footerData',e)



    json_data = json.dumps(json_data_dict)  
    return json_data
    # else:

    #     for i in range(0,len(queryset_detail)):
    #         for j in field_query_detail:
    #             data = str(queryset_detail[i][j[0]])
    #             if data.find('.')>0:
    #                 index = data.find('.')
    #                 queryset_detail[i][j[0]] = data[0:index+1+j[1]]+j[2]


# def accuratelyNumber(v):
#     v = v.replace('.',',')
#     index = v.find(',')
#     if index > 0:
#         sub_v = v[0:index]   #parte entera            
#         if len(sub_v)>3: 
#             new_name = ''
#             i = 0
#             for t in range(len(sub_v),0,-1):
#                 t = t-1
#                 new_name = sub_v[t] +new_name 
#                 i = i+1
#                 if (i%3)==0 and i!=0:
#                     new_name =  '.'+new_name 
#             v = new_name+v[index:]
#         if v[0] == '.':
#             v = v[1:]
#     return v


def filterFormat(new_queryset):
    dict_type_data={}
    try:

        if len(new_queryset)>0:
            for k,v in new_queryset[0].items():
                if (type(v) is not unicode) and (type(v) is not datetime): 
                    dict_type_data [k] = type(v)
    except Exception,e:

        swslog('debug','List index out range',e)
        dict_type_data=False

    return dict_type_data   


def numberFormatter(number):
  # Nos aseguramos de que sea un string
  a = str(number)
  # Buscamos la posicion donde esta el separador de decimales
  p = a.find('.')
  # Guardamos el string con los decimales sustituyendo el punto por una coma (si es que hay)
  if p == -1:
    decimales = ''
  else:
    decimales = a[p:].replace('.',',')
    # Cogemos el entero
    a = a[:p]
  # Variables auxiliares
  b = ''
  c = ''
  i = 0
  # Generamos el numero/string con los miles
  while i < len(a)/3:
    b = a[len(a)-(3*(i+1)):len(a)-(len(c)-i)] 
    c = '.'+b + c
    i+=1
  # Concatenamos los decimales al resultado
  c = a[:len(a)-(3*i)] + c + decimales
  # Devolvemos el resultado

  if c[0]=='.':
    c = c[1:]

  c = c.replace('.',',')

  return c






def getTableDict(json_data, title_dict, value_group_by_data):
    # table_dict = SortedDict({'titles_table':title_dict})
    json_data_dict = ujson.loads(json_data)  
    data = json_data_dict['rows']

    first_element = True
    for key,value in title_dict.items():
        if first_element == True:
            title_dict.pop(key)
            title_dict.insert(0,value_group_by_data,value_group_by_data.capitalize())
            first_element = False

    # table_dict = SortedDict({'titles_table':title_dict})
    cont = 0
    series_dict = SortedDict()
    for d in data:
        exists = False
        for key,value in series_dict.items():
            if value_group_by_data == 'ALL':
                value_group_by_data = 'Client'
            if(value[value_group_by_data] == d[value_group_by_data]):
                row_dict = SortedDict()
                for v in value:
                    element = series_dict.get(key)
                    if v == value_group_by_data:
                        element_value = d[v]
                    elif v == 'ASR':
                        # try:
                        #     element_value = float(element[v]) + float(d[v])
                        #     element_value = str(round(element_value,3))
                        # except:
                        element_value = str(float(element[v]) + float(d[v]))

                    elif v == 'ACD':
                        element_value = str(float(element[v]) + float(d[v]))
                    elif v == 'answer_calls':

                        # element_value = element[v]
                        if value['ASR'] == '0.0' or value['Calls'] == '0.0':
                            element_value = '0'
                        else:
                            if float (value['Calls']) != 0:
                                element_value = str(float(value['ASR']) / float(value['Calls'])) 
                            else:
                                element_value = '0';
                    elif v == 'calls':
                        element_value = str(int(element[v]) + int(d[v]))
                    

                    row_dict[v] = element_value
                
                series_dict[serie] = row_dict
                exists = True

        if not(exists):
            cont = cont + 1
            serie = 'serie' + str(cont)

            row_dict = SortedDict()
            for e in title_dict.keys():
                if e == 'answer_calls':
                    row_dict[e] = '0'
                else:
                    if e == 'ALL':
                        e = 'Client'


                    row_dict[e] = d[e]
            series_dict[serie] = row_dict